<?php

namespace Insign\PostCore\Block\Adminhtml\System\Config;

use Insign\PostCore\Helper\BarcodeHelper;
use Magento\Backend\Block\Template\Context;

class AdditionalServiceCheckboxes extends \Magento\Config\Block\System\Config\Form\Field
{
    protected $values = null;

    /**
     * @var BarcodeHelper
     */
    protected $barcodeHelper;
    /**
     * @var \Insign\PostCore\Model\Config\Source\Barcode\AdditionalServiceCheckBoxes
     */
    protected $serviceCheckBoxes;

    protected $template = 'Insign_PostCore::system/config/checkbox.phtml';

    /**
     * AdditionalServiceCheckboxes constructor.
     * @param BarcodeHelper $barcodeHelper
     * @param \Insign\PostCore\Model\Config\Source\Barcode\AdditionalServiceCheckBoxes $serviceCheckBoxes
     * @param Context $context
     * @param array $data
     */
    public function __construct(
        BarcodeHelper $barcodeHelper,
        \Insign\PostCore\Model\Config\Source\Barcode\AdditionalServiceCheckBoxes $serviceCheckBoxes,
        Context $context,
        array $data = []
    ) {
        $this->barcodeHelper = $barcodeHelper;
        $this->serviceCheckBoxes = $serviceCheckBoxes;
        $this->setTemplate($this->template);
        parent::__construct($context, $data);
    }

    protected function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element): string
    {
        $this->setNamePrefix($element->getName())
            ->setHtmlId($element->getHtmlId());
        return $this->_toHtml();
    }

    public function getValues(): array
    {
        $values = [];
        foreach ($this->serviceCheckBoxes->toOptionArray() as $value) {
            $values[$value['value']] = $value['label'];
        }

        return $values;
    }

    public function getIsChecked($name): bool
    {
        return in_array($name, $this->getCheckedValues());
    }

    public function getCheckedValues(): array
    {
        if (is_null($this->values)) {
            $data = $this->barcodeHelper->getAdditionalService();
            if (isset($data)) {
                $this->values = $data;
            } else {
                $this->values = '';
            }
        }

        return $this->values;
    }
}
