<?php

namespace Insign\PostCore\Controller\Barcode;

use Insign\PostCore\Model\BarcodeFactory;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Url\DecoderInterface;

class Popup extends \Magento\Framework\App\Action\Action
{

    /**
     * Allowed hash keys
     *
     * @var array
     */
    protected $_allowedHashKeys = ['entity_id'];

    /**
     * @var BarcodeFactory
     */
    protected $barcodeFactory;

    /**
     * @var DecoderInterface
     */
    protected $urlDecoder;

    /**
     * @var Session
     */
    protected $sessionBarcode;

    /**
     * @param Context $context
     * @param BarcodeFactory $barcodeFactory
     * @param DecoderInterface $urlDecoder
     * @param Session $sessionBarcode
     */
    public function __construct(
        Context $context,
        BarcodeFactory $barcodeFactory,
        DecoderInterface $urlDecoder,
        Session $sessionBarcode
    ) {
        $this->barcodeFactory = $barcodeFactory;
        $this->urlDecoder = $urlDecoder;
        $this->sessionBarcode = $sessionBarcode;
        parent::__construct($context);
    }

    /**
     * Popup action
     * Shows tracking info if it's present, otherwise redirects to 404
     *
     * @return void
     * @throws NotFoundException
     */
    public function execute()
    {
        $barcodeInfo = $this->decodeHash($this->getRequest()->getParam('hash'));
        if (count($barcodeInfo) == 0) {
            throw new NotFoundException(__('Page not found.'));
        }
        if (isset($barcodeInfo['id'])) {
            $this->sessionBarcode->setVarValue($barcodeInfo['id']);
        }
        $this->_view->loadLayout();
        $this->_view->getPage()->getConfig()->getTitle()->set(__('Swiss Post Barcode'));
        $this->_view->renderLayout();
    }

    /**
     * @param $hash
     * @return array
     */
    public function decodeHash($hash): array
    {
        $hash = explode(':', $this->urlDecoder->decode($hash));
        if (count($hash) === 3 && in_array($hash[0], $this->_allowedHashKeys)) {
            return ['key' => $hash[0], 'id' => (int)$hash[1], 'hash' => $hash[2]];
        }
        return [];
    }
}
