<?php

namespace Insign\PostCore\Helper;

use Insign\PostCore\Api\TokenRepositoryInterface;
use Insign\PostCore\Logger\Log;
use Insign\PostCore\Model\TokenFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;

class TokenHelper
{
    const XML_SWISS_POST_ADDRESS_VALIDATION_ENABLE = 'swisspost/address_validation/validation_active';
    const XML_SWISS_POST_ADDRESS_AUTOCOMPLETION_ENABLE = 'swisspost/address_autocompletion/address_autocompletion_active';
    const XML_SWISS_POST_CLIENT_ID = 'swisspost/authentication/client_id';
    const XML_SWISS_POST_CLIENT_SECRET = 'swisspost/authentication/client_secret';
    const XML_SWISS_POST_TOKEN_ENDPOINT = 'swisspost/authentication/token_endpoint';

    const XML_SWISS_POST_ADDRESS_VALIDATION_ENDPOINT = 'swisspost/address_validation/endpoint';
    const XML_SWISS_POST_ADDRESS_VALIDATION_SCOPE = 'swisspost/address_validation/scope';

    const XML_SWISS_POST_ADDRESS_AUTOCOMPLETE_STREET = 'swisspost/address_autocompletion/street_name_endpoint';
    const XML_SWISS_POST_ADDRESS_AUTOCOMPLETE_ZIP = 'swisspost/address_autocompletion/zip_endpoint';
    const XML_SWISS_POST_ADDRESS_AUTOCOMPLETE_SCOPE = 'swisspost/address_autocompletion/scope';

    /** @var ScopeConfigInterface  */
    protected $scopeConfig;

    /** @var \Insign\PostCore\Helper\TokenRepositoryInterface  */
    protected $tokenRepository;

    /** @var tokenFactory  */
    protected $tokenFactory;

    /** @var Log  */
    protected $logger;

    public function __construct(
        ScopeConfigInterface $scopeConfig,
        TokenRepositoryInterface $tokenRepository,
        TokenFactory $tokenFactory,
        Log $logger
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->tokenRepository = $tokenRepository;
        $this->tokenFactory = $tokenFactory;
        $this->logger = $logger;
    }

    /**
     * @return bool
     */
    public function isValidationEnable(): bool
    {
        $status = $this->scopeConfig->getValue(
            $this::XML_SWISS_POST_ADDRESS_VALIDATION_ENABLE,
            ScopeInterface::SCOPE_WEBSITE
        );
        if ($status > 0) {
            return true;
        }
        return false;
    }

    /**
     * @return bool
     */
    public function isAutocompletionEnable(): bool
    {
        $status = $this->scopeConfig->getValue(
            $this::XML_SWISS_POST_ADDRESS_AUTOCOMPLETION_ENABLE,
            ScopeInterface::SCOPE_WEBSITE
        );
        if ($status > 0) {
            return true;
        }
        return false;
    }

    /** @return mixed */
    public function getClientId()
    {
        return $this->scopeConfig->getValue(
            $this::XML_SWISS_POST_CLIENT_ID,
            ScopeInterface::SCOPE_WEBSITE
        );
    }

    /** @return mixed */
    public function getClientSecret()
    {
        return $this->scopeConfig->getValue(
            $this::XML_SWISS_POST_CLIENT_SECRET,
            ScopeInterface::SCOPE_WEBSITE
        );
    }

    /** @return mixed */
    public function getTokenEndpoint()
    {
        return $this->scopeConfig->getValue(
            $this::XML_SWISS_POST_TOKEN_ENDPOINT,
            ScopeInterface::SCOPE_WEBSITE
        );
    }

    /** @return mixed */
    public function getAddressValidationEndpoint()
    {
        return $this->scopeConfig->getValue(
            $this::XML_SWISS_POST_ADDRESS_VALIDATION_ENDPOINT,
            ScopeInterface::SCOPE_WEBSITE
        );
    }

    /** @return mixed */
    public function getAddressValidationScope()
    {
        return $this->scopeConfig->getValue(
            $this::XML_SWISS_POST_ADDRESS_VALIDATION_SCOPE,
            ScopeInterface::SCOPE_WEBSITE
        );
    }

    /** @return mixed */
    public function getAddressAutocompleteStreet()
    {
        return $this->scopeConfig->getValue(
            $this::XML_SWISS_POST_ADDRESS_AUTOCOMPLETE_STREET,
            ScopeInterface::SCOPE_WEBSITE
        );
    }

    /** @return mixed */
    public function getAddressAutocompleteZip()
    {
        return $this->scopeConfig->getValue(
            $this::XML_SWISS_POST_ADDRESS_AUTOCOMPLETE_ZIP,
            ScopeInterface::SCOPE_WEBSITE
        );
    }

    /** @return mixed */
    public function getAddressAutocompleteScope()
    {
        return $this->scopeConfig->getValue(
            $this::XML_SWISS_POST_ADDRESS_AUTOCOMPLETE_SCOPE,
            ScopeInterface::SCOPE_WEBSITE
        );
    }
}
