<?php

namespace Insign\PostCore\Model;

use Insign\PostCore\Api\AddressAutocompleteInterface;
use Insign\PostCore\Helper\TokenHelper;
use Insign\PostCore\Util\OAuth2\AbstractProvider;
use Insign\PostCore\Util\OAuth2\AccessTokenManager;
use Insign\PostCore\Util\OAuth2\AddressAutocompleteProvider;
use Magento\Framework\App\Config\ScopeConfigInterface;

class StreetAutocompleteManagement implements AddressAutocompleteInterface
{
    /** @var int  */
    protected $requestCounter = 0;

    /** @var AccessTokenManager  */
    protected $accessTokenManager;

    /** @var ScopeConfigInterface  */
    protected $scopeConfig;

    /** @var AddressAutocompleteProvider  */
    protected $oauth2Provider;

    /** @var TokenHelper  */
    protected $tokenHelper;

    /**
     * StreetAutocompleteManagement constructor.
     * @param AccessTokenManager $accessTokenManager
     * @param TokenHelper $tokenHelper
     * @param AddressAutocompleteProvider $oauth2Provider
     */
    public function __construct(
        AccessTokenManager $accessTokenManager,
        TokenHelper $tokenHelper,
        AddressAutocompleteProvider $oauth2Provider
    ) {
        $this->accessTokenManager = $accessTokenManager;
        $this->tokenHelper = $tokenHelper;
        $this->oauth2Provider = $oauth2Provider;
    }

    /**
     * @param string $streetName
     * @return array
     * @throws \Exception
     */
    public function autocompleteField(string $streetName): array
    {
        if (!$this->tokenHelper->isAutocompletionEnable()) {
            return false;
        }

        $options = [];
        $response = [];

        $url = $this->tokenHelper->getAddressAutocompleteStreet();

        $options['query'] = [
            'name' => $streetName
        ];

        $accessToken = $this->accessTokenManager->getAccessToken($this->tokenHelper->getAddressAutocompleteScope());
        if (!isset($accessToken)) {
            $accessToken = $this->oauth2Provider->createAccessToken();
        }
        try {
            $responseJson = $this->oauth2Provider->createAuthenticatedRequest(AbstractProvider::METHOD_HTTP_GET, $url, $accessToken, $options);

            if($responseJson->getStatusCode() == 401 && $this->requestCounter < 1){
                $this->requestCounter++;
                $this->oauth2Provider->createAccessToken();
                return $this->autocompleteField($streetName);
            }
            elseif ($responseJson->getStatusCode() == 401 && $this->requestCounter == 1){
                $this->logger->error("Post API Response: " . $responseJson->getStatusCode());
                throw new Exception("Post API Response: " . $responseJson->getStatusCode());
            }

            $response = json_decode((string)$responseJson->getBody(), true);
            if (isset($response['streets'])) {
                return $response['streets'];
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401 && $this->requestCounter < 1) {
                $this->requestCounter++;
                $this->oauth2Provider->createAccessToken();
                return $this->autocompleteField($streetName);
            }
        }
        return $response;
    }
}
