<?php

namespace Insign\PostCore\Model;

use Insign\PostCore\Api\AddressAutocompleteInterface;
use Insign\PostCore\Helper\TokenHelper;
use Insign\PostCore\Util\OAuth2\AccessTokenManager;
use Insign\PostCore\Util\OAuth2\AbstractProvider;
use Insign\PostCore\Util\OAuth2\AddressAutocompleteProvider;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;

class ZipAutocompleteManagement implements AddressAutocompleteInterface
{
    /**
     * @var int
     */
    private $requestCounter = 0;

    /**
     * @var AccessTokenManager
     */
    protected $accessTokenManager;
    /**
     * @var AddressAutocompleteProvider
     */
    protected $oauth2Provider;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var TokenHelper
     */
    private $tokenHelper;

    /**
     * ZipAutocompleteManagement constructor.
     * @param AccessTokenManager $accessTokenManager
     * @param TokenHelper $tokenHelper
     * @param AddressAutocompleteProvider $oauth2Provider
     */
    public function __construct(
        AccessTokenManager $accessTokenManager,
        TokenHelper $tokenHelper,
        AddressAutocompleteProvider $oauth2Provider
    ) {
        $this->accessTokenManager = $accessTokenManager;
        $this->tokenHelper = $tokenHelper;
        $this->oauth2Provider = $oauth2Provider;
    }


    /**
     * @param string $zipCity
     * @return array
     * @throws \Exception
     */
    public function autocompleteField(string $zipCity): array
    {
        if (!$this->tokenHelper->isAutocompletionEnable()) {
            return false;
        }

        $options = [];
        $response = [];


        $url = $this->tokenHelper->getAddressAutocompleteZip();
        $options['query'] = [
            'type' => AddressAutocompleteInterface::AUTOCOMPLETE_TYPE_DOMICILE,
            'zipCity' => $zipCity
        ];
        $accessToken = $this->accessTokenManager->getAccessToken($this->tokenHelper->getAddressAutocompleteScope());
        if (!isset($accessToken)) {
            $accessToken = $this->oauth2Provider->createAccessToken();
        }

        try {
            $responseJson = $this->oauth2Provider->createAuthenticatedRequest(AbstractProvider::METHOD_HTTP_GET, $url, $accessToken, $options);

            if($responseJson->getStatusCode() == 401 && $this->requestCounter < 1){
                $this->requestCounter++;
                $this->oauth2Provider->createAccessToken();
                return $this->autocompleteField($zipCity);
            }
            elseif ($responseJson->getStatusCode() == 401 && $this->requestCounter == 1){
                $this->logger->error("Post API Response: " . $responseJson->getStatusCode());
                return "ERROR";
            }

            $response = json_decode((string)$responseJson->getBody(), true);
            if (isset($response['zips'])) {
                return $response['zips'];
            }
        } catch (\Exception $e) {
            if ($e->getCode() == 401 && $this->requestCounter < 1) {
                $this->requestCounter++;
                $this->oauth2Provider->createAccessToken();
                return $this->autocompleteField($zipCity);
            } else {
                return "ERROR";
            }
        }
        return $response;
    }
}
